#!/usr/bin/env python3
# (C) Copyright 2022 European Centre for Medium-Range Weather Forecasts.
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
from __future__ import annotations

import climetlab as cml
from climetlab import Dataset
from climetlab.decorators import normalize
from climetlab.readers.csv import CSVReader

__version__ = "0.3.0"


class Main(Dataset):
    name = "Surface Observation Postprocessing dataset"
    home_page = "-"
    # The licence is the licence of the data (not the licence of the plugin)
    licence = "-"
    documentation = "-"
    citation = "-"

    # These are the terms of use of the data (not the licence of the plugin)
    terms_of_use = (
        "By downloading data from this dataset, "
        "you agree to the terms and conditions defined at "
        "https://github.com/mchantry/"
        "climetlab-mltc-sop/"
        "blob/main/LICENSE. "
        "If you do not agree with such terms, do not download the data. "
    )

    dataset = None

    @normalize("field", ["forecast_error", "time_of_day", "soil_temperature"])
    def __init__(self, field):
        URL = "https://get.ecmwf.int/repository/mooc-machine-learning-weather-climate/"
        PATTERN = "{url}/{field}.csv"
        request = dict(field=field, url=URL)
        try:
            self.source = cml.load_source(
                "url", PATTERN.format(**request), reader=CSVReader
            )
        except:  # noqa
            URL = "https://object-store.os-api.cci1.ecmwf.int"
            request = dict(field=field, url=URL)
            PATTERN = "{url}/sop/{field}.csv"
            self.source = cml.load_source(
                "url", PATTERN.format(**request), reader=CSVReader
            )

    def to_numpy(self):
        return self.to_pandas().to_numpy()
